from xdxsmi.api.node import Node
from pathlib import Path
import re, glob

from xdxsmi.api.test2 import *

#Acquire frame buffer and system memory info
def read_mem_info():
    try:
        fo = open("/sys/kernel/debug/xdxgpu/gpu00/debug_dump")
        #[\s\S]*GPU_LOCAL, Type: LMA, CPU PA Base: 0x[0-9]*, GPU PA Base: 0x[0-9]*, Usage Flags: 0x[0-9]*, Refs: 10, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*
        #[\s\S]*SYSMEM, Type: UMA, Usage Flags: 0x[0-9]*, Refs: 1, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*
        res = re.match( r'[\s\S]*GPU_LOCAL[\s\S]*10, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*SYSMEM, Type: UMA, Usage Flags: 0x[0-9]*, Refs: 1, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*', fo.read())
        if res == None:
            res = ['N/A', 'N/A', 'N/A', 'N/A', 'N/A']
        fo.close()
    except:
        res = ['N/A', 'N/A', 'N/A', 'N/A', 'N/A']
    return res

#Acquire statistics of pcie errors
def read_err_corr(file: str):
    res = re.match(r'[\s\S]*TOTAL_ERR_COR ([0-9]*)[\s\S]*', file)
    return res[1] if res!=None else 'N/A'

def read_err_fatal(file: str):
    res = re.match(r'[\s\S]*TOTAL_ERR_FATAL ([0-9]*)[\s\S]*', file)
    return res[1] if res!=None else 'N/A'

def read_err_nonfat(file: str):
    res = re.match(r'[\s\S]*TOTAL_ERR_NONFATAL ([0-9]*)[\s\S]*', file)
    return res[1] if res!=None else 'N/A'

#Match PCIE link speed with PCIE link generation
def speed_2_gen(speed: str):
    speed = float(speed)
    if speed == 64:
        return "6.0"
    elif speed == 32 or speed == 25:
        return "5.0"
    elif speed == 16:
        return "4.0"
    elif speed == 8:
        return "3.0"
    elif speed == 5:
        return "2.0"
    else:
        return "1.0"

#Match PCIE link generation with PCIE throughput
def gen_2_throughput(gen:str):
    if gen == "6":
        return "0.250 GB/s"
    elif gen == "5":
        return "0.500 GB/s"
    elif gen == "4":
        return "0.985 GB/s"
    elif gen == "3":
        return "1.969 GB/s"
    elif gen == "2":
        return "3.938 GB/s"
    elif gen == "1":
        return "7.563 GB/s"
    else:
        return "N/A"

# #Acquire attributes in each directory
# class XdxGpu():
#     def __init__(self, index) -> None:
#         self.drm_index = index
#         self.card = Node(f"/sys/class/drm/card{index}")
#         #self.debugfs = Node(f"/sys/kernel/debug/dri/{index}")
#         #self.debugfs = Node(f"/sys/kernel/debug/xdxgpu")
#         self.device = Node(f"/sys/class/drm/card{index}/device")
#         self.dsp_type = ''
#         self.dsp_mode = 'disconnected'
#         try:
#             #hdmi_path = next(Path(f"/sys/class/drm/card{index}/").glob("card*"))
#             display_status_paths = glob.glob(f"/sys/class/drm/card{index}/card*/status")
#             for path in display_status_paths:
#                 fo = open(path)
#                 status = fo.readline().strip('\n')
#                 if status == 'connected':
#                     self.dsp_mode = status
#                     self.dsp_type = 'HDMI' if path.find('DP') == -1 else 'DP'
#                 fo.close()
#         except:
#             self.dsp_mode = 'N/A'

#         try:
#             hwmon_path = next(Path(f"/sys/class/drm/card{index}/device/hwmon")\
#                      .glob("hwmon*"))
#             self.hwmon = Node(hwmon_path)
#         except StopIteration:
#             self.hwmon = None
        

#         #GPU realted attributes xdx_vbios
#         self.name = self.card['xdx_product_name'] if self.card['xdx_product_name']!=None else 'N/A'
#         self.core = self.hwmon['xdx_gpu_core'] if self.hwmon['xdx_gpu_core']!=None else 'N/A'
#         self.primary_id = '0x' + self.hwmon['xdx_gpu_primary_id'] if self.hwmon['xdx_gpu_primary_id']!=None else 'N/A'
#         self.gddr_count = '0x' + self.hwmon['xdx_gddr_count'] if self.hwmon['xdx_gddr_count']!=None else 'N/A'
#         self.pcie_mmbar_type = self.hwmon['xdx_pcie_mmbar_type'] if self.hwmon['xdx_pcie_mmbar_type']!=None else 'N/A'
#         self.asci_type = self.hwmon['xdx_asci_type'] if self.hwmon['xdx_asci_type']!=None else 'N/A'
#         #self.dsp_mode = self.dsp_mode if self.dsp_mode!=None else 'N/A'
#         self.acct_mode = 'N/A'
#         self.uuid = '0x' + self.hwmon['xdx_chip_id'] if self.hwmon['xdx_chip_id']!=None else 'N/A'
#         self.minor_num = index

#         try:
#             self.vbios = self.hwmon['xdx_vbios'] if self.hwmon['xdx_vbios']!=None else 'N/A'
#         except:
#             self.vbios = 'N/A'

#         self.fm_ver = self.hwmon['xdx_fw_ver'] if self.hwmon['xdx_fw_ver']!=None else 'N/A'

#         self.pci_bdf = self.card['xdx_dbdf'] if self.card['xdx_dbdf']!=None else 'N/A'
#         self.domain = '0x' + self.pci_bdf.split(':',3)[0] if self.pci_bdf!='N/A' else 'N/A'
#         self.bus = '0x' + self.pci_bdf.split(':',3)[1] if self.pci_bdf!='N/A' else 'N/A'
#         self.dev = '0x' + self.pci_bdf.split(':',3)[2].split('.',1)[0] if self.pci_bdf!='N/A' else 'N/A'
#         self.device_id = self.device['device'] if self.device['device']!=None else 'N/A'
#         self.subsystem_device = self.device['subsystem_device'] if self.device['subsystem_device']!=None else 'N/A'

#         self.primary_core_id = self.hwmon['xdx_gpu_primary_core_id'] if self.hwmon['xdx_gpu_primary_core_id']!=None else 'N/A'

#         self.pcie_event = self.hwmon['xdx_pcie_event'] if self.hwmon['xdx_pcie_event']!=None else 'N/A'
#         self.perstAssert = True if(self.pcie_event != "N/A" and int(self.pcie_event) & 1) else False
#         self.perstDeAssert = True if(self.pcie_event != "N/A" and int(self.pcie_event) & 2) else False
#         self.linkClkStable = True if(self.pcie_event != "N/A" and int(self.pcie_event) & 4) else False
#         self.linkDown = True if(self.pcie_event != "N/A" and int(self.pcie_event) & 8) else False
#         self.perstAssertAcpuAck = True if(self.pcie_event != "N/A" and int(self.pcie_event) & 16) else False
#         self.perstDeAssertAcpuAck = True if(self.pcie_event != "N/A" and int(self.pcie_event) & 32) else False

#         #PCIE Generation and Width  
#         gen_max = self.device['max_link_speed']
#         self.pcie_gen_max = speed_2_gen(gen_max[0:gen_max.find(' GT/s')]) if self.device['max_link_speed']!=None else 'N/A'

#         gen_curr = self.device['current_link_speed']
#         self.pcie_gen_cur = speed_2_gen(gen_curr[0:gen_curr.find(' GT/s')]) if self.device['current_link_speed']!=None else 'N/A'

#         self.throughput = gen_2_throughput(self.pcie_gen_cur)

#         self.width_max = 'x' + self.device['max_link_width'] if self.device['max_link_width']!=None else 'N/A'
#         self.width_cur = 'x' + self.device['current_link_width'] if self.device['current_link_width']!=None else 'N/A'

#         #Acquire values for pice error related attributes
#         if self.device['aer_dev_correctable']!=None:
#             self.pcie_err_corr = read_err_corr(self.device['aer_dev_correctable'])
#         else:
#             self.pcie_err_corr = 'N/A'

#         if self.device['aer_dev_fatal']!=None:
#             self.pcie_err_fatal = read_err_fatal(self.device['aer_dev_fatal'])
#         else:
#             self.pcie_err_fatal = 'N/A'

#         if self.device['aer_dev_nonfatal']!=None:
#             self.pcie_err_nonfat = read_err_nonfat(self.device['aer_dev_nonfatal'])
#         else:
#             self.pcie_err_nonfat = 'N/A'

#         self.fan_spd = self.hwmon['fan1_input']+'RPM' if self.hwmon['fan1_input']!=None else 'N/A'
#         self.perform_state = 'N/A'
#         self.clk_throt_reasons = 'N/A'
#         self.mem = read_mem_info()

#         #GPU memory info
#         self.fb_free = 'N/A' if self.mem[1]=='N/A' else int(int(self.mem[1]) / (1024*1024))
#         self.fb_total = 'N/A' if self.mem[2]=='N/A' else int(int(self.mem[2]) / (1024*1024))
#         self.fb_used = 'N/A' if self.fb_total=='N/A' or self.fb_free=='N/A' else int(self.fb_total - self.fb_free)
        
#         #Host memory info
#         self.sys_free = 'N/A' if self.mem[3]=='N/A' else int(int(self.mem[3]) / (1024*1024))
#         self.sys_total = 'N/A' if self.mem[4]=='N/A' else int(int(self.mem[4]) / (1024*1024))
#         self.sys_used = 'N/A' if self.sys_total=='N/A' or self.sys_free=='N/A' else int(self.sys_total - self.sys_free)

#         self.mem_util = 'N/A'
#         self.ecc_curr = 'N/A'
#         self.ecc_pend = 'N/A'
#         self.ecc_error = 'N/A'
#         self.temp = self.hwmon['temp1_input'] + "\u00B0" + "C" if self.hwmon['temp1_input']!=None else 'N/A'
#         self.pwr_read = 'N/A'
#         self.gfx_clk = str(int(self.hwmon['xdx_gfx_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_gfx_freq_mhz']!=None else 'N/A'
#         self.acpu_clk = str(int(self.hwmon['xdx_acpu_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_acpu_freq_mhz']!=None else 'N/A'
#         #self.host_clk = str(int(self.hwmon['xdx_host_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_host_freq_mhz']!=None else 'N/A'
#         self.ndf_clk = str(int(self.hwmon['xdx_ndf_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_ndf_freq_mhz']!=None else 'N/A'
#         self.gddr_clk = str(int(self.hwmon['xdx_gddr_phys_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_gddr_phys_freq_mhz']!=None else 'N/A'
#         self.cmcu_clk = str(int(self.hwmon['xdx_cmcu_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_cmcu_freq_mhz']!=None else 'N/A'
#         self.vid_clk = 'N/A'
#         self.app_clk = 'N/A'
#         self.def_app_clk = 'N/A'
#         self.sup_clk = 'N/A'

#     def __setitem__(self, name, val):
#         return setattr(self, name, val)

#     def __getitem__(self, name):
#         return getattr(self, name)



class XdxGpu():
    def __init__(self, index, user_index, handle) -> None:
        #print("zihan_debug:enter XdxGpu")
        #xdxml_init()
        #handle = xdxml_device_get_handle_by_index(index)
        queryInfo = xdxml_device_query(handle) 

        #Yufei think index(id) doesn't match with handle(device address)
        # 2023-8-30: index has been updated to the true index of card 
        self.drm_index = index
        self.user_index = user_index      # for user
        self.card = Node(f"/sys/class/drm/card{index}")
        #self.debugfs = Node(f"/sys/kernel/debug/dri/{index}")
        #self.debugfs = Node(f"/sys/kernel/debug/xdxgpu")
        self.device = Node(f"/sys/class/drm/card{index}/device")
        self.dsp_type = ''
        self.dsp_mode = 'disconnected'
        self.handle = handle
        
        try:
            display_status_paths = glob.glob(f"/sys/class/drm/card{index}/card*/status")
            for path in display_status_paths:
                fo = open(path, 'r')
                status = fo.read()
                
                # do not use queryInfo.dsp_mode
                # which is the wrong result when DP/HDMI is more than 1
                if status == 'connected\n':
                    self.dsp_mode = 'connected'
                    if self.dsp_type == '':
                        # the content in file node to store HDMI and DP status has a '\n' in the tail
                        if (path.find('DP') == -1):
                            self.dsp_type = ' HDMI '
                        elif (path.find('HDMI') == -1):
                            self.dsp_type = '  DP  '
                        else:
                            self.dsp_type = ''
        
                fo.close()
        except:
            self.dsp_mode = 'N/A'
        try:
            hwmon_path = next(Path(f"/sys/class/drm/card{index}/device/hwmon")\
                     .glob("hwmon*"))
            self.hwmon = Node(hwmon_path)
        except StopIteration:
            self.hwmon = None

        #GPU realted attributes xdx_vbios
        self.name = queryInfo.name.decode('utf-8') if queryInfo.name!=None else 'N/A'
        # show core agian, and get the core number by ioctl
        self.core = queryInfo.core if queryInfo.core != None else 'N/A'
        self.primary_id = '0x' + queryInfo.primary_id.decode('utf-8') if queryInfo.primary_id!=None else 'N/A'
        self.gddr_count = '0x' + str(queryInfo.gddr_count) if queryInfo.gddr_count!=None else 'N/A'
        self.gddr_speed = str(queryInfo.gddr_speed/1e9) + 'Gb/s' if queryInfo.gddr_speed!=0 else 'N/A'
        self.gddr_size = str(queryInfo.gddr_size) + 'GB' if queryInfo.gddr_size != None else 'N/A'
        self.pcie_mmbar_type = queryInfo.mmbar_type.decode('utf-8') if queryInfo.mmbar_type!=None else 'N/A'
        self.asci_type = queryInfo.asci_type.decode('utf-8') if queryInfo.asci_type!=None else 'N/A'
        #self.dsp_mode = self.dsp_mode if self.dsp_mode!=None else 'N/A'
        self.acct_mode = 'N/A'
        self.uuid = '0x' + str(queryInfo.uuid.decode('utf-8')) if queryInfo.uuid!=None else 'N/A'
        self.minor_num = index

        try:
            self.vbios = queryInfo.vbios_version.decode('utf-8').split(' ',1)[0] if queryInfo.vbios_version!=None else 'N/A'
            #self.vbios = self.hwmon['xdx_vbios'].split(' ',1)[0] if self.hwmon['xdx_vbios']!=None else 'N/A'
        except:
            self.vbios = 'N/A'

        self.fm_ver = queryInfo.fw_ver.decode('utf-8') if queryInfo.fw_ver!=None else 'N/A'

        self.pci_bdf = queryInfo.pci_dbdf.decode('utf-8') if queryInfo.pci_dbdf!=None else 'N/A'
        self.domain = '0x' + self.pci_bdf.split(':',3)[0] if self.pci_bdf!='N/A' else 'N/A'
        self.bus = '0x' + self.pci_bdf.split(':',3)[1] if self.pci_bdf!='N/A' else 'N/A'
        self.dev = '0x' + self.pci_bdf.split(':',3)[2].split('.',1)[0] if self.pci_bdf!='N/A' else 'N/A'
        self.device_id = self.device['device'] if self.device['device']!=None else 'N/A'
        self.subsystem_device = self.device['subsystem_device'] if self.device['subsystem_device']!=None else 'N/A'

        self.primary_core_id = queryInfo.primary_core_id.decode('utf-8') if queryInfo.primary_id!=None else 'N/A'

        self.pcie_event = queryInfo.pcie_event if queryInfo.pcie_event!=None else 'N/A'
        self.perstAssert = True if(self.pcie_event != "N/A" and queryInfo.perst_assert) else False
        self.perstDeAssert = True if(self.pcie_event != "N/A" and queryInfo.perst_de_assert) else False
        self.linkClkStable = True if(self.pcie_event != "N/A" and queryInfo.link_clk_stable) else False
        self.linkDown = True if(self.pcie_event != "N/A" and queryInfo.link_down) else False
        self.perstAssertAcpuAck = True if(self.pcie_event != "N/A" and queryInfo.perst_assert_acpu_ack) else False
        self.perstDeAssertAcpuAck = True if(self.pcie_event != "N/A" and queryInfo.perst_de_assert_acpu_ack) else False

        #PCIE Generation and Width  
        gen_max = self.device['max_link_speed']
        self.pcie_gen_max = queryInfo.max_link_gen if queryInfo.max_link_gen!=None else 'N/A'
        gen_curr = self.device['current_link_speed']
        self.pcie_gen_cur = str(queryInfo.curr_link_gen) if queryInfo.curr_link_gen!=None else 'N/A'

        self.throughput = gen_2_throughput(self.pcie_gen_cur)

        self.width_max = 'x' + str(round(queryInfo.max_pcie_link_width, 1)) if queryInfo.max_pcie_link_width!=None else 'N/A'
        self.width_cur = 'x' + str(queryInfo.curr_pcie_link_width) if queryInfo.curr_pcie_link_width!=None else 'N/A'

        #Acquire values for pice error related attributes
        if self.device['aer_dev_correctable']!=None:
            self.pcie_err_corr = queryInfo.err_corr.decode('utf-8')
        else:
            self.pcie_err_corr = 'N/A'

        if self.device['aer_dev_fatal']!=None:
            self.pcie_err_fatal = queryInfo.err_fatal.decode('utf-8')
        else:
            self.pcie_err_fatal = 'N/A'

        if self.device['aer_dev_nonfatal']!=None:
            self.pcie_err_nonfat = queryInfo.non_fatal.decode('utf-8')
        else:
            self.pcie_err_nonfat = 'N/A'
        self.support_fan_info = int(queryInfo.support_fan_info)
        if self.support_fan_info:
            # the file node of pwm1 has been discarded, cause it is redundant with fan_spd
            self.fan_spd = self.hwmon['fan1_input'] if self.hwmon['fan1_input']!=None else 'N/A'
        else :
            self.fan_spd = 'N/A'

        self.perform_state = 'N/A'
        self.clk_throt_reasons = 'N/A'
        #self.mem = read_mem_info()

        #GPU memory info
        self.fb_free = 'N/A' if queryInfo.fb_free =='N/A' else int(int(queryInfo.fb_free) / (1024*1024))
        self.fb_total = 'N/A' if queryInfo.fb_total =='N/A' else int(int(queryInfo.fb_total) / (1024*1024))
        self.fb_used = 'N/A' if self.fb_total=='N/A' or self.fb_free=='N/A' else int(self.fb_total - self.fb_free)
        
        #Host memory info
        self.sys_free = 'N/A' if queryInfo.sys_free =='N/A' else int(int(queryInfo.sys_free) / (1024*1024))
        self.sys_total = 'N/A' if queryInfo.sys_total =='N/A' else int(int(queryInfo.sys_total) / (1024*1024))
        self.sys_used = 'N/A' if self.sys_total=='N/A' or self.sys_free=='N/A' else int(self.sys_total - self.sys_free)

        self.mem_util = 'N/A'
        self.ecc_curr = 'N/A'
        self.ecc_pend = 'N/A'
        self.ecc_error = 'N/A'
        self.temp = str(queryInfo.temp) + "\u00B0" + "C" if queryInfo.temp!=None else 'N/A'
        self.pwr_read = 'N/A'
        self.gfx_clk = str(int(queryInfo.xdx_gfx_freq/1e6)) +'MHZ' if queryInfo.xdx_gfx_freq!=None else 'N/A'
        self.acpu_clk = str(int(queryInfo.xdx_acpu_freq/1e6)) +'MHZ' if queryInfo.xdx_acpu_freq!=None else 'N/A'
        #self.host_clk = str(int(self.hwmon['xdx_host_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_host_freq_mhz']!=None else 'N/A'
        self.ndf_clk = str(int(queryInfo.xdx_ndf_freq/1e6)) +'MHZ' if queryInfo.xdx_ndf_freq!=None else 'N/A'
        self.gddr_clk = str(int(queryInfo.xdx_gddr_phys_freq/1e6)) +'MHZ' if queryInfo.xdx_gddr_phys_freq!=None else 'N/A'
        self.cmcu_clk = str(int(queryInfo.xdx_cmcu_freq/1e6)) +'MHZ' if queryInfo.xdx_cmcu_freq!=None else 'N/A'
        self.vid_clk = 'N/A'
        self.app_clk = 'N/A'
        self.def_app_clk = 'N/A'
        self.sup_clk = 'N/A'

        #wcj

        self.bValid = True if(queryInfo.bValid !=0 ) else False

        self.ui64GpuStatActive = queryInfo.ui64GpuStatActive
        self.ui64GpuStatBlocked = queryInfo.ui64GpuStatBlocked
        self.ui64GpuStatIdle = queryInfo.ui64GpuStatIdle
        self.ui64GpuStatCumulative = queryInfo.ui64GpuStatCumulative

        self.aaui64DMOSStatActive_0 = queryInfo.aaui64DMOSStatActive_0
        self.aaui64DMOSStatBlocked_0 = queryInfo.aaui64DMOSStatBlocked_0
        self.aaui64DMOSStatIdle_0 = queryInfo.aaui64DMOSStatIdle_0
        self.aaui64DMOSStatCumulative_0 = queryInfo.aaui64DMOSStatCumulative_0

        self.aaui64DMOSStatActive_1 = queryInfo.aaui64DMOSStatActive_1
        self.aaui64DMOSStatBlocked_1 = queryInfo.aaui64DMOSStatBlocked_1
        self.aaui64DMOSStatIdle_1 = queryInfo.aaui64DMOSStatIdle_1
        self.aaui64DMOSStatCumulative_1 = queryInfo.aaui64DMOSStatCumulative_1
        self.twoDTDMusage = 'N/A' if  queryInfo.aaui64DMOSStatCumulative_1 == 0 else format(queryInfo.aaui64DMOSStatActive_1*100/queryInfo.aaui64DMOSStatCumulative_1,'.3f')+'%'

        self.aaui64DMOSStatActive_2 = queryInfo.aaui64DMOSStatActive_2
        self.aaui64DMOSStatBlocked_2 = queryInfo.aaui64DMOSStatBlocked_2
        self.aaui64DMOSStatIdle_2 = queryInfo.aaui64DMOSStatIdle_2
        self.aaui64DMOSStatCumulative_2 = queryInfo.aaui64DMOSStatCumulative_2
        self.GEOMMusage = 'N/A' if  queryInfo.aaui64DMOSStatCumulative_2 == 0 else format(queryInfo.aaui64DMOSStatActive_2*100/queryInfo.aaui64DMOSStatCumulative_2,'.3f')+'%'

        self.aaui64DMOSStatActive_3 = queryInfo.aaui64DMOSStatActive_3
        self.aaui64DMOSStatBlocked_3 = queryInfo.aaui64DMOSStatBlocked_3
        self.aaui64DMOSStatIdle_3 = queryInfo.aaui64DMOSStatIdle_3
        self.aaui64DMOSStatCumulative_3 = queryInfo.aaui64DMOSStatCumulative_3
        self.threeDMusage = 'N/A' if  queryInfo.aaui64DMOSStatCumulative_3 == 0 else format(queryInfo.aaui64DMOSStatActive_3*100/queryInfo.aaui64DMOSStatCumulative_3,'.3f')+'%'

        self.aaui64DMOSStatActive_4 = queryInfo.aaui64DMOSStatActive_4
        self.aaui64DMOSStatBlocked_4 = queryInfo.aaui64DMOSStatBlocked_4
        self.aaui64DMOSStatIdle_4 = queryInfo.aaui64DMOSStatIdle_4
        self.aaui64DMOSStatCumulative_4 = queryInfo.aaui64DMOSStatCumulative_4
   
        self.aaui64DMOSStatActive_5 = queryInfo.aaui64DMOSStatActive_5
        self.aaui64DMOSStatBlocked_5 = queryInfo.aaui64DMOSStatBlocked_5
        self.aaui64DMOSStatIdle_5 = queryInfo.aaui64DMOSStatIdle_5
        self.aaui64DMOSStatCumulative_5 = queryInfo.aaui64DMOSStatCumulative_5
   
        self.aaui64DMOSStatActive_6 = queryInfo.aaui64DMOSStatActive_6
        self.aaui64DMOSStatBlocked_6 = queryInfo.aaui64DMOSStatBlocked_6
        self.aaui64DMOSStatIdle_6 = queryInfo.aaui64DMOSStatIdle_6
        self.aaui64DMOSStatCumulative_6 = queryInfo.aaui64DMOSStatCumulative_6
    
        self.aaui64DMOSStatActive_7 = queryInfo.aaui64DMOSStatActive_7
        self.aaui64DMOSStatBlocked_7 = queryInfo.aaui64DMOSStatBlocked_7
        self.aaui64DMOSStatIdle_7 = queryInfo.aaui64DMOSStatIdle_7
        self.aaui64DMOSStatCumulative_7 = queryInfo.aaui64DMOSStatCumulative_7

        self.aaui64DMOSStatActive_8 = queryInfo.aaui64DMOSStatActive_8
        self.aaui64DMOSStatBlocked_8 = queryInfo.aaui64DMOSStatBlocked_8
        self.aaui64DMOSStatIdle_8 = queryInfo.aaui64DMOSStatIdle_8
        self.aaui64DMOSStatCumulative_8 = queryInfo.aaui64DMOSStatCumulative_8

        self.ui64TimeStamp = queryInfo.ui64TimeStamp

        self.pronum = queryInfo.pronum

        self.pro1PID = queryInfo.process_1        
        self.pro1LMA = format(queryInfo.pro1LMA/(1024*1024),'.3f')
        self.pro1Total = format(queryInfo.pro1Total/(1024*1024),'.3f')

        self.pro2PID = queryInfo.process_2        
        self.pro2LMA = format(queryInfo.pro2LMA/(1024*1024),'.3f')
        self.pro2Total = format(queryInfo.pro2Total/(1024*1024),'.3f')

        self.pro3PID = queryInfo.process_3        
        self.pro3LMA = format(queryInfo.pro3LMA/(1024*1024),'.3f')
        self.pro3Total = format(queryInfo.pro3Total/(1024*1024),'.3f')

        self.pro4PID = queryInfo.process_4        
        self.pro4LMA = format(queryInfo.pro4LMA/(1024*1024),'.3f')
        self.pro4Total = format(queryInfo.pro4Total/(1024*1024),'.3f')

        self.pro5PID = queryInfo.process_5        
        self.pro5LMA = format(queryInfo.pro5LMA/(1024*1024),'.3f')
        self.pro5Total = format(queryInfo.pro5Total/(1024*1024),'.3f')

        self.pro1NAME = queryInfo.pro1NAME.decode('utf-8')
        self.pro2NAME = queryInfo.pro2NAME.decode('utf-8')
        self.pro3NAME = queryInfo.pro3NAME.decode('utf-8')
        self.pro4NAME = queryInfo.pro4NAME.decode('utf-8')
        self.pro5NAME = queryInfo.pro5NAME.decode('utf-8')


    def __setitem__(self, name, val):
        return setattr(self, name, val)

    def __getitem__(self, name):
        return getattr(self, name)
